# [Working with Factors](@id work_w_factors)

Factors represent the interaction between particular variables. Factors define the algebra structure between variables via probabilistic measurement models as captured measurement or data cues.  For example, a distance travelled measurement between two pose variables. Relative factors between variables are probabilistic models that capture the likelihood interactions between variables. Priors factors (i.e. unary to one variable) represent absolute information to be introduced about that variable, for example a GPS measurement; more on how to introduce distrust of such priors later.

## List or Get Factors

All factors in a graph can be listed via:
```julia
flbls = listFactors(client, context) |> fetch
# ["x0f_8ebc", ...]

# and for a naturally ordered list
flbls = sort(flbls; lt=NvaSDK.natural_lt)
```

Note the factor labels are autogenerated based on the variables and a random sequence.  For example a prior factor might have the label `x0f_8ebc`, while factor between three variables might be `x7x49l4f_654t`.

A specific factor can be retrieved using the label:
```julia
f = getFactor(client, context, flbls[1]) |> fetch
# Dict{String, Any} with 8 entries:
#   "label"                 => "x0f_8ebc"
#   "_variableOrderSymbols" => Any["x0"]
#   "data"                  => "eyJlbGltaW5hdGVkIjpmYWxzZSwicG90ZW50aWFsdXNlZCI6Z…
#   "tags"                  => Any["FACTOR"]
#   "timestamp"             => "2023-02-11T22:36:49.171Z"
#   "_version"              => "0.18.6"
#   "fnctype"               => "PriorPose2"
#   "solvable"              => 1
```

```@docs
listFactors
getFactor
```

## App Graph Visualization

The NavAbilityApp can also visualize a specific factor graph session, and a convenient link generator can be used to simplify the session filtering criteria:
```julia
GraphVizApp(context, variableStartsWith="")
# https://app.navability.io/cloud/graph/?userId=guest@navability.io&robotStartsWith=SDKjl_0181&sessionStartsWith=Tutorial1_af33&variableStartsWith
```

A `variableStartsWith="l"` type filter can be used to filter the visualization to more specific elements of the graph.  This is useful in large graphs where the visualization easily overwhelms the display -- e.g. just show variables starting with `l` which may indicate landmarks.

!!! note
    A console output will only produce the text URL as above, but Markdown enhanced output (such as Jupyter or Pluto notebooks) will also produce a clickable graphic.

The NavAbilityApp Graph visualization page can also be found in the hamburger menu (top right).  Once on the Graph visualization page, the global filter menu (top left) can be used to look for specific user, robot, session, and starts with criteria.  Here is an example following from earlier (as per one of the ICRA 2022 tutorials) showing a variable `x0` and the unary prior factor `x0f_8ebc`:

```@raw html
<a href="https://app.navability.io/cloud/graph/?userId=guest@navability.io&robotStartsWith=SDKjl_0181&sessionStartsWith=Tutorial1_af33&variableStartsWith">
<p align="center">
<img src="https://user-images.githubusercontent.com/6412556/218284321-83fad03b-ea54-4a0a-976d-49020f48dcd4.png" width="480" border="0" />
</p>
</a>
```

```@docs
GraphVizApp
```

## SDK Supported Factors

The list of currently supported factors in the SDK are:
- `Prior`, for 1D Euclidean only.
- `LinearRelative`, for Euclidean only.
- `Point2Point2Range`
- `PriorPoint2`
- `PriorPose2`
- `Pose2Pose2`
- `ScatterAlignPose2`
- `Pose2AprilTag4Corners`
- `Pose2Point2BearingRange`
- `PriorPose3`
- `Pose3Pose3`

!!! note
    Many more factor types are already supported by the solver, see [additional docs here](https://juliarobotics.org/Caesar.jl/latest/concepts/available_varfacs/).  Reach out to NavAbility for help or support in bringing more variable types to eh SDK sooner, or for help in building more factor types that may not yet exist in either libraries.

## Factor Index

```@docs
NvaSDK.ZInferenceType
NvaSDK.InferenceType
NvaSDK.PriorData
NvaSDK.LinearRelativeData
NvaSDK.PriorPoint2Data
NvaSDK.Point2Point2RangeData
NvaSDK.PriorPose2Data
NvaSDK.Pose2Pose2Data
NvaSDK.Pose2Point2BearingRangeData
NvaSDK.PriorPose3Data
NvaSDK.Pose3Pose3Data
NvaSDK.ScatterAlignPose2
NvaSDK.Pose2AprilTag4Corners
```